/* 
 * File:   main.c
 * Author: Pavel
 *
 * Created on 15. leden 2019, 18:39
 * 
 * XC8 linker - Memory model - ROM ranges -1FFE-1FFF   
 */

#include <xc.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "Dipm.h"
#include "oled1106.c"

#pragma config FOSC = ECH    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin) 
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled) 
#pragma config PWRTE = ON      // Power-up Timer Enable (PWRT disabled) 
#pragma config MCLRE = OFF       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR) 
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled) 
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled) 
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin) 

// CONFIG2 
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off) 
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset) 
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.) 
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled) 
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming) 
#pragma config PPS1WAY = 0FF 

void DispN(char n) {
    char len;
    itoa(strt, n, 10);
    len = strlen(strt);
    if (len == 2) {
        strt[2] = strt[1];
        strt[1] = strt[0];
        strt[0] = 0x20;
    }
    if (len == 1) {
        strt[2] = strt[0];
        strt[1] = 0x20;
        strt[0] = 0x20;
    }

    cursorXY(curs = 90, line = 6);
    TextB(strt);
}

void MemRead() {
    char adrl = 0xE0;
    PMADRH = 0x1F; //Store MSB of address
    for (char i = 0; i < 32; i++) {
        PMADRL = adrl; //Store LSB of address    
        PMCON1bits.CFGS = 0; //Do not select Configuration Space
        PMCON1bits.RD = 1; //Initiate read
        asm("NOP"); //Ignored (Figure 10-2)
        asm("NOP"); //Ignored (Figure 10-2)
        ebuf[i] = PMDATL;
        adrl++;
    }
}

void Unlock() {
    PMCON2 = 0x55; // Write 55h
    PMCON2 = 0xAA; //Write AAh
    PMCON1bits.WR = 1; // Set WR bit to begin write
    asm("NOP");
    asm("NOP");
}

void MemWrite() {
    char adrl = 0xE0;
    GIE = 0; //Disable ints so required sequences will execute properly
    PMADRH = 0x1F; //Store MSB of address
    for (char i = 0; i < 32; i++) {
        PMADRL = adrl;
        PMCON1bits.CFGS = 0; //Do not select Configuration Space
        PMCON1bits.FREE = 0; //not Specify an erase operation
        PMCON1bits.LWLO = 1; // Only Load Write Latches
        PMCON1bits.WREN = 1; //Initiate Enable writes
        PMDATL = ebuf[i]; //*dat;
        PMDATH = 0;
        //  dat++;
        adrl++;
        Unlock();
    }

    PMCON1bits.LWLO = 0; //No more loading latches - Actually start Flash program
    Unlock();
    PMCON1bits.WREN = 0; //Disable writes
}

void MemEras() {
    char adr1 = 0xE0;
    GIE = 0; //Disable ints so required sequences will execute properly
    PMADRL = adr1;
    PMADRH = 0x1F;
    PMCON1bits.CFGS = 0; //Not configuration space
    PMCON1bits.FREE = 1; //Specify an erase operation
    PMCON1bits.WREN = 1; //Enable writes
    Unlock();
    PMCON1bits.WREN = 0; //Disable writes

}

inline void setModul(void) {
    char dif = 11-Modul;
    if(dif==0)dif=1;
    mampl = Bias / dif;
}



void main(void) {

    PORTA = 0;
    PORTC = 0;
    LATA = 0;
    LATC = 0;
    TRISA = 0xFC;
    TRISC = 0x3F; //RC5 in

    ANSELA = 0x0; // 
    ANSELC = 0x00;
    ANSC3 = 1; //RC3 
    WPUA = 0x0;
    WPUC = 0x0;
    OPTION_REGbits.nWPUEN = 0;
    INTEDG = 0;
    //OPTION_REGbits.PS=7; //prescaler /2   
    OSCCON = 0xF0; //0x78;  32MHz
    //OSCTUNE = 0x01;

    ADCON0 = 0x0D; //AN3
    ADCON1 = 0x63; //0xE3; // po?te?n nastaven
    ADCON2 = 0x0F; //fosc/64, 
    FVRCONbits.ADFVR0 = 1; //ref = FVR
    FVREN = 1;




    /*---------------- Peripheral Pin Select (PPS) -------------------*/
    GIE = 0;
    PPSLOCK = 0x55;
    PPSLOCK = 0xAA;
    PPSLOCKED = 0; //enable PPS

    RC0PPS = 0x10; //RC0 jako SCLK 
    RC1PPS = 0x11; //RC1 jako SDA  
    INTPPS = 0x12; //RC2 jako INTE
    T1CKIPPS = 0x14; //RC4 jako T1CLK

    PPSLOCKED = 1; //dis. PPS
    /*-------------------------------------------------*/
    MemRead();
    if (ebuf[3] > 3) {
        Bias = 66;
        Thre = 87;
        Light = 20;
        Arng = 1;
        Modul = 0;
        Mode = 0;
        Menu = 0;
        memcpy(ebuf, rbuf, 8);
        memcpy(ebuf + 8, rbuf, 8);
        memcpy(ebuf + 16, rbuf, 8);
        memcpy(ebuf + 24, rbuf, 8);
        MemEras();
        MemWrite();
        GIE = 1;
    } else memcpy(rbuf, ebuf, 8);

    INTE = 1;
    Delay(300);
    OpenI2C();
    initDAC();
    Delay(500);
    initDisplay();

    PR2 = 249;
    // T2CON = 0x27; // 
    T2CONbits.T2CKPS = 3; //prescaler = 64
    T2CONbits.T2OUTPS = 4; //postscaler = 5
    TMR2ON = 1;

    T1CON = 0x84;
    T1GCON = 0; //init TMR1 
    TMR2IE = 1;
    TMR1IE = 1;

    PEIE = 1;
    GIE = 1;

    DACEN = 1;
    DAC1OE2 = 1;
    DAC1CON1 = Bias;
    FVRCON = 0x8C + Arng;
    tflg = 0;

    while (!tflg) {
        freq = tml;
        FreqCnt(tml);
    }

    Delay(100);
    Graph4(4, dac / 32);
    cursorXY(curs = 0, line = 6);
    TextB(tmode[Mode & 3]);
    WriteDAC();
    WritePOT();

    while (1) {

        if (enc) {
            //  enc = 0;
            if ((emen & 7) == 0) {
                if (!updn) {
                    dac += fi;
                    if (dac > 0x0FFF) dac = 0xFFF;
                } else if (updn) {
                    dac -= fi;
                    if (dac < 0)dac = 0;
                }
                WriteDAC();
                Graph4(4, dac / 32);
            } else if ((emen & 7) == 3) {
                if (Menu == 6) { //   bias  
                    if (!updn) {
                        if (Bias < 0xFF)
                            Bias++;
                    } else if (updn) {
                        if (Bias) Bias--;
                    }
                    DAC1CON1 = Bias; setModul();
                    DispN(Bias);
                } else if (Menu == 7) { //thres         
                    if (!updn) {
                        if (Thre < 0xFF)
                            Thre++;
                    } else if (updn) {
                        if (Thre) Thre--;
                    }
                    WritePOT();
                    DispN(Thre);
                } else if (Menu == 8) { //Arng
                    if (!updn) {
                        if (Arng < 3)
                            Arng++;
                    } else if (updn) {
                        if (Arng > 1) Arng--;
                    }
                    DispN(Arng);
                } else if (Menu == 9) { //modul
                    if (!updn) {
                        if (Modul < 10)
                            Modul++;
                    } else if (updn) {
                        if (Modul) Modul--;
                    }
                    setModul();
                    DispN(Modul);
                } else if (Menu == 10) { //light
                    if (!updn) {
                        if (Light < 0x7F)
                            Light++;
                    } else if (updn) {
                        if (Light) Light--;
                    }
                    setContrast(Light);
                    DispN(Light);
                } else if (Menu == 12 || Menu == 11) { //save / read
                    Save &= 3;
                    if (!updn) {
                        if (Save < 3)
                            Save++;
                    } else if (updn) {
                        if (Save > 1) Save--;
                    }
                    DispN(Save & 3);
                }
            } else if ((emen & 7) == 1) {
                if (!updn) {
                    if (Menu < 14)
                        Menu++;

                } else if (updn) {
                    if (Menu > 0) Menu--;
                }
                cursorXY(curs = 80, line = 6);
                TextB(tmenu[Menu & 0x0F]);
            }
            enc = 0;
        }//enc
        if (PORTAbits.RA3 == 1)fine &= ~4;
        if (PORTAbits.RA3 == 0 && (fine & 4) == 0) {
            fine++;
            fine &= 3;
            fine |= 4;
            cursorXY(curs = 80, line = 6);
            TextB(tfine[fine & 3]);            
        }//fine

        if (PORTCbits.RC5 == 1)emen &= ~8;
        if (PORTCbits.RC5 == 0 && (emen & 8) == 0) {
            if(scan){stop = 1; }
            else{
            emen++;
            if ((emen & 7) == 1) {
                cursorXY(curs = 80, line = 6);
                TextB(tmenu[Menu & 0x0F]);
            }

            if ((emen & 7) == 2) {
                emen = 4;
                switch (Menu & 0x0F) {
                    case 0: //emen = 4;
                        break;
                    case 1: Mode = 0;
                        break;
                    case 2: Mode = 1;
                        break;
                    case 3: Mode = 2;
                        break;
                    case 4: Mode = 3;
                        break;
                    case 5: scan = 1; stop=0; 
                        dac = min;
                        WriteDAC();
                        umin = 0xFF;
                        cursorXY(curs = 60, line = 6);
                        TextB("S");
                        break;
                    case 13: min = dac;
                    break;
                    case 14: max = dac;
                    break;

                    default: emen = 3;
                        if (Menu == 6) DispN(Bias);
                        else if (Menu == 7) DispN(Thre);
                        else if (Menu == 8) DispN(Arng);
                        else if (Menu == 9) DispN(Modul);
                        else if (Menu == 10) DispN(Light);
                        else if (Menu == 11 || Menu == 12) DispN(Save);
                }
            }

            if ((emen & 7) == 4) {
                MemRead(); //na?te do ebuf
                if (Menu == 11 || Menu == 12) {
                    Save &= 3;
                    if (Menu == 12) {
                        memcpy(rbuf, ebuf + (8 * Save), 8);
                        setContrast(Light);
                        DAC1CON1 = Bias;
                        WriteDAC();
                        WritePOT();
                    } else {
                        MemEras();
                        memcpy(ebuf + (8 * Save), rbuf, 8);
                        MemWrite();
                        GIE = 1;
                    }
                }
                else {                   
                    MemEras();
                    memcpy(ebuf, rbuf, 8);
                    MemWrite();
                    GIE = 1;
                }
                cursorXY(curs = 0, line = 6);
                TextB(tmode[Mode & 3]);
                cursorXY(curs = 80, line = 6);
                TextB(tfine[fine & 3]);
                emen = 0;
            }
        }
            emen |= 8;
        }//klik 

         switch (fine & 3) {
            case 0: fi = 32;
                break;
            case 1: fi = 16;
                break;
            case 2: fi = 8;
                break;
            case 3: fi = 1;
                break;
        }
        
        if(Modul){
            mflag ^= 1;
            if(mflag)
            DAC1CON1 = Bias+ mampl;
            else DAC1CON1 = Bias - mampl; 
        }
        else DAC1CON1 = Bias;

        if (Mode & 1) {
            PORTAbits.RA1 = 1;
        } else {
            PORTAbits.RA1 = 0;
        }

        if (Mode & 2) {
            PORTAbits.RA0 = 1;
        } else {
            PORTAbits.RA0 = 0;
        }

        if (tflg) {
            freq = tml;
            FreqCnt(tml);
            // if(graf>freq && dac<255)dac++; 
            // if(graf<freq && dac>0)dac--;
            //  StopI2C();
            cursorXY(curs = 0, line = 2);

           
            TextB(str);
 
        }


        // Arng = 1;
        FVRCON = (char) (0x8C + Arng);
        ADCON0 = 0x1D; // AN7, ADON

        GO = 1;
        while (GO);
        if (ADRESL & 1)uadc = 0;
        else uadc = ADRESH;
        //  if(uadc==0xFF && arng<2)arng++;
        //  if(uadc<127 && arng>0)arng--;
        if (uadc == 0)uadc = 3;

        Graph2(0, uadc / 2);
        if (scan && (tf & 2)) {           
            if (uadc < umin) {
                umin = uadc;
                dacmin = dac;
            }

            dac += fi;
            if (dac > max || stop) { //dac = 0xFFF;
                scan = 0; stop = 0;
                dac = dacmin;               
                cursorXY(curs = 60, line = 6);
                TextB(" ");
            }
            WriteDAC();
            Graph4(4, dac / 32);

        }

    };
}//main

char FreqCnt(unsigned long x) {


    char len = 0, slen = 0; //row;
    char frac = '0';
    tflg = 0;
    char hilo = 1;

    if (Mode & 1)hilo = 0;
    ultoa(str, tml /*>> 2*/, 10);

    len = strlen((const char *) str);

    if (hilo && len < 6) {
        str[0] = '?';
        str[1] = 0;
    } else {
        //  if ((char) tml & 2)frac = '5';

        if (len > 2) {
            str[len] = str[len - 1];
            str[len - 1] = str[len - 2];
            str[len - 2] = str[len - 3];

            if (len > (5 - hilo)) {
                str[len - 3] = str[len - 4];
                if (hilo == 0) str[len - 4] = str[len - 5];
                str[len - 5 + hilo] = '.';
                len -= 3;
                str[len + 3] = 'M';
            }//if len>5
            else {
                str[len - 2 ] = '.';
                str[len + 3] = 'k';
            }

            str[len + 2] = 0x7F;
            strcpy(&str[len + 4], "Hz"); //Hz

        }//if len>2
        else {
            if (tml > 0) {
                str[len] = frac;
            }//if tml
            else len--;

            str[len + 1] = 0x7F;
            strcpy(&str[len + 2], "Hz"); //Hz
        }
    }
    len = strlen((const char *) str);
    while ((len) <= 11) {
        str[len] = 0x7F;
        len++;
    }

    str[len] = 0;
    return slen;
}


void interrupt ISBR() {

    if (INTF) {
        INTF = 0;
        if (enc == 0) {
            if (PORTAbits.RA4 == 0) {
                updn = 1;
            } else {
                updn = 0;
            }
            enc = 1;
        }
    }

    if (TMR2IF) {
        TMR2IF = 0;
        //    if (freq) {
        tf++;
        if (tf == 20)
            tf = 0;
        if (tf == 1 /*|| tf == 21 || tf == 41 || tf==61 || tf==81*/) {
            //  
            //   TMR1L = TMR1H = tup = 0;
            TMR1ON = 1; //otevre Tmr1
        } else if (tf == 11/*|| tf == 31 || tf== 51 || tf==71 || tf==91*/) {
            //Sdelay(3);
            asm("NOP");
            asm("NOP");
            asm("NOP");
            TMR1ON = 0; // zavre Tmr1  
            tml = TMR1L;

            *((char *) &tml + 1) = TMR1H;
            *((unsigned int *) &tml + 1) = tup;
            TMR1L = TMR1H = tup = 0;

            tflg = 1;
            // };
        };

        /*  count++;
          if (count >= 20) {
              count = 0;
              binp = 1;
          }*/

    }//if tmr2if

    if (TMR1IF) {
        tup++;
        TMR1IF = 0;
    }
};
